package ij.plugin;

import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.Arrow;
import ij.gui.DialogListener;
import ij.gui.GenericDialog;
import ij.gui.NonBlockingGenericDialog;
import ij.gui.Roi;
import ij.gui.Toolbar;
import java.awt.AWTEvent;
import java.awt.Color;

import javax.swing.JFrame;
import ij.plugin.frame.PlugInFrame;
import java.util.Vector;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import ij.util.Tools;
import javax.swing.JScrollBar;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;


/** This plugin implements the Edit/Options/Arrow Tool command. */
public class ArrowToolOptions implements PlugIn, DialogListener, DocumentListener, AdjustmentListener, ItemListener  {
        private String colorName;
        private static GenericDialog gd;
        private static String title = "Arrow Tool";
        private static JFrame frame;
        private Vector numbers, sliders, choices, checkboxes;

 	public void run(String arg) {
 		if (gd!=null && gd.isVisible())
 			gd.toFront();
 		else
			arrowToolOptions();
	}

	void arrowToolOptions() {
		if (!Toolbar.getToolName().equals("arrow"))
			IJ.setTool("arrow");
		double width = Arrow.getDefaultWidth();
		double headSize = Arrow.getDefaultHeadSize();
		Color color = Toolbar.getForegroundColor();
		colorName = Colors.getColorName(color, "red");
		int style = Arrow.getDefaultStyle();
                gd = frame!=null?new NonBlockingGenericDialog(title, frame):new NonBlockingGenericDialog(title);
		//gd = new NonBlockingGenericDialog("Arrow Tool");
		gd.addSlider("Width:", 1, 50, (int)width);
		gd.addSlider("Size:", 0, 30, headSize);
		gd.addChoice("Color:", Colors.colors, colorName);
		gd.addChoice("Style:", Arrow.styles, Arrow.styles[style]);
		gd.addCheckbox("Outline", Arrow.getDefaultOutline());
		gd.addCheckbox("Double head", Arrow.getDefaultDoubleHeaded());
		gd.addDialogListener(this);
                numbers = gd.getNumericFields();
                for (int i=0; i<numbers.size(); i++)
                    ((JTextField)numbers.elementAt(i)).getDocument().addDocumentListener(this);
                sliders = gd.getSliders();
                for (int i=0; i<sliders.size(); i++)
                    ((JScrollBar)sliders.elementAt(i)).addAdjustmentListener(this);
                choices = gd.getChoices();
                for (int i=0; i<choices.size(); i++)
                    ((JComboBox)choices.elementAt(i)).addItemListener(this);
                checkboxes = gd.getCheckboxes();
		gd.showDialog();
	}

	public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
            double width2 = Tools.parseDouble(((JTextField)numbers.elementAt(0)).getText());
            double headSize2 = Tools.parseDouble(((JTextField)numbers.elementAt(1)).getText());
            String colorName2 = (String)((JComboBox)choices.elementAt(0)).getSelectedItem();
            int style2 = ((JComboBox)choices.elementAt(1)).getSelectedIndex();
            boolean outline2 = ((JCheckBox)checkboxes.elementAt(0)).isSelected();
            boolean doubleHeaded2 = ((JCheckBox)checkboxes.elementAt(1)).isSelected();
		if (colorName!=null && !colorName2.equals(colorName)) {
			Color color = Colors.getColor(colorName2, Color.black);
			Toolbar.setForegroundColor(color);
		}
		colorName = colorName2;
		Arrow.setDefaultWidth(width2);
		Arrow.setDefaultHeadSize(headSize2);
		Arrow.setDefaultStyle(style2);
		Arrow.setDefaultOutline(outline2);
		Arrow.setDefaultDoubleHeaded(doubleHeaded2);
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) return true;
		Roi roi = imp.getRoi();
		if (roi==null) return true;
		if (roi instanceof Arrow) {
			Arrow arrow = (Arrow)roi;
			roi.setStrokeWidth((float)width2);
			arrow.setHeadSize(headSize2);
			arrow.setStyle(style2);
			arrow.setOutline(outline2);
			arrow.setDoubleHeaded(doubleHeaded2);
			imp.draw();
		}
		Prefs.set(Arrow.STYLE_KEY, style2);
		Prefs.set(Arrow.WIDTH_KEY, width2);
		Prefs.set(Arrow.SIZE_KEY, headSize2);
		Prefs.set(Arrow.OUTLINE_KEY, outline2);
		Prefs.set(Arrow.DOUBLE_HEADED_KEY, doubleHeaded2);
		return true;
	}
        public void textValueChanged(DocumentEvent e) {
            double width2 = Tools.parseDouble(((JTextField)numbers.elementAt(0)).getText());
            double headSize2 = Tools.parseDouble(((JTextField)numbers.elementAt(1)).getText());
            String colorName2 = (String)((JComboBox)choices.elementAt(0)).getSelectedItem();
            int style2 = ((JComboBox)choices.elementAt(1)).getSelectedIndex();
            boolean outline2 = ((JCheckBox)checkboxes.elementAt(0)).isSelected();
            boolean doubleHeaded2 = ((JCheckBox)checkboxes.elementAt(1)).isSelected();
            if (colorName!=null && !colorName2.equals(colorName)) {
                    Color color = Colors.getColor(colorName2, Color.black);
                    Toolbar.setForegroundColor(color);
            }
            colorName = colorName2;
            Arrow.setDefaultWidth(width2);
            Arrow.setDefaultHeadSize(headSize2);
            Arrow.setDefaultStyle(style2);
            Arrow.setDefaultOutline(outline2);
            Arrow.setDefaultDoubleHeaded(doubleHeaded2);
            ImagePlus imp = WindowManager.getCurrentImage();
            if (imp==null) return ;
            Roi roi = imp.getRoi();
            if (roi==null) return ;
            if (roi instanceof Arrow) {
                    Arrow arrow = (Arrow)roi;
                    roi.setStrokeWidth((float)width2);
                    arrow.setHeadSize(headSize2);
                    arrow.setStyle(style2);
                    arrow.setOutline(outline2);
                    arrow.setDoubleHeaded(doubleHeaded2);
                    imp.draw();
            }
            Prefs.set(Arrow.STYLE_KEY, style2);
            Prefs.set(Arrow.WIDTH_KEY, width2);
            Prefs.set(Arrow.SIZE_KEY, headSize2);
            Prefs.set(Arrow.OUTLINE_KEY, outline2);
            Prefs.set(Arrow.DOUBLE_HEADED_KEY, doubleHeaded2);
        }

        public void insertUpdate(DocumentEvent e){
            textValueChanged(e);
        }
        public void removeUpdate(DocumentEvent e){
            textValueChanged(e);
        }
        public void changedUpdate(DocumentEvent e){
        }
        public synchronized void adjustmentValueChanged(AdjustmentEvent e) {
                Object source = e.getSource();
                for (int i=0; i<sliders.size(); i++) {
                        if (source==sliders.elementAt(i)) {
                                JScrollBar sb = (JScrollBar)source;
                                JTextField tf = (JTextField)numbers.elementAt(i);
                                textValueChanged(null);
                        }
                }
        }
        public synchronized void itemStateChanged(ItemEvent e) {
                Object source = e.getSource();
                for (int i=0; i<choices.size(); i++) {
                        if (source==choices.elementAt(i)) {
                                JComboBox sb = (JComboBox)source;
                                textValueChanged(null);
                        }
                }
        }


}
