package ij2x.plugin;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.Undo;

import ij.gui.GenericDialog;
import ij.gui.ImageCanvas;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.gui.Toolbar;

import ij.process.ImageProcessor;

import ij.plugin.filter.PlugInFilter;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Polygon;
import java.awt.geom.AffineTransform;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import java.util.Vector;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.JFormattedTextField;
import javax.swing.text.*;
import ij.plugin.PlugIn;

/** This plugin implements the Image/Rotate/Arbitrarily command. */
public class RoiRotator implements PlugIn, DocumentListener, ChangeListener {
    private static double angle = 0.0;
    private double oldangle=0.0, curAngle=0.0;
    private boolean interpolate = true;
    private static boolean fillWithBackground;
    private boolean enlarge;
    private Vector<JComponent> slider, spinner;
    private JFormattedTextField textField;
    private JSlider jslider;
    private JSpinner jspinner;
    private ImagePlus imp, clipBoard;
    private Roi roi ;
    private ImageProcessor ip;
    private GenericDialog gd;
    private Graphics g;
    private Document doc;
    private Vector<JComponent> textfield;
    private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    private Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
    private double mag=1.0;
    protected Rectangle srcRect;
    private ImageCanvas ic;

        public void run(String arg) {
            imp = IJ.getImage();
            ic = imp.getCanvas();
            srcRect = ic.getSrcRect();
            g=ic.getGraphics();
            int bitDepth = imp.getBitDepth();
            ip = imp.getProcessor();
            roi = imp.getRoi();
            mag = imp.getCanvas().getMagnification();
            //if (mag<1.0)
                if (!showDialog(imp, bitDepth))
                        return;
                int slices = imp.getStackSize();
                boolean rotateStack = false;
                long startTime = System.currentTimeMillis();
                boolean zeroFill = false;

                if (slices>1 && !enlarge) {
                        int result = IJ.setupDialog(imp, 0);
                        if (result==PlugInFilter.DONE)
                                return;
                        rotateStack = result==PlugInFilter.DOES_STACKS;
                }
                if (slices==1 && !enlarge) {
                        ip.snapshot();
                        Undo.setup(Undo.FILTER, imp);
                } else if (slices>1)
                        Undo.reset();
                IJ.showStatus("Rotating...");
                if (!rotateStack) slices = 1;
                int slice = imp.getCurrentSlice();
                ImageStack stack = imp.getStack();
                for (int i=1; i<=slices; i++) {
                        if (rotateStack) {
                                IJ.showProgress(i, slices);
                                IJ.showStatus("Rotating... ("+i+"/"+slices+")");
                                ip = stack.getProcessor(i);
                        }
                        ip.setInterpolate(interpolate);
                        if (fillWithBackground) {
                                Color bgc = Toolbar.getBackgroundColor();
                                if (bitDepth==8)
                                        ip.setBackgroundValue(ip.getBestIndex(bgc));
                                else if (bitDepth==24)
                                        ip.setBackgroundValue(bgc.getRGB());
                        } else {
                                if (zeroFill)
                                        ip.setBackgroundValue(0);
                                else {
                                        if (bitDepth==8 || bitDepth==24)
                                                ip.setColor(Color.white);
                                }
                        }
                }
                if (rotateStack) {
                        imp.setStack(null, stack);
                        imp.setSlice(slice);
                }
                imp.updateAndDraw();
                imp.changes = true;
                if (enlarge && slices==1)
                        Undo.setup(Undo.COMPOUND_FILTER_DONE, imp);
                IJ.showTime(imp, startTime, "Rotate: ");
        }

        boolean showDialog(ImagePlus imp, int bitDepth) {
                        //roi = imp.getRoi();
                        Rectangle r = roi!=null?roi.getBounds():null;
                        boolean canEnlarge = r==null || (r.x==0&&r.y==0&&r.width==imp.getWidth()&&r.height==imp.getHeight());
                        gd = new GenericDialog("Rotate", IJ.getInstance());
                        String angleLabel="Angle:";
                        int min=0, max=360;
                        int mior=90/2;
                        int major=90;
                        int width=164, height=58;
                        boolean paintticks=true, paintLabels=true;
                        JLabel transformLabel = new JLabel(" Rotate ");
                        transformLabel.setBorder(grayBorder);
                        JPanel transformPanel = new JPanel();
                        transformPanel.add(transformLabel);
                        gd.addPanel(transformPanel, GridBagConstraints.WEST, new Insets(0, 0, 0, 10));
                        gd.addJSlider(angleLabel, min, max, angle, major, mior, paintticks, paintLabels, width, height);
                        spinner=gd.getSpinners();
                        slider =gd.getSliders();
                        textfield = gd.getSpinnerFields();
                        jspinner=(JSpinner) spinner.elementAt(0);
                        jslider=(JSlider) slider.elementAt(0);
                        textField=(JFormattedTextField) textfield.elementAt(0);
                        jslider.addChangeListener(this);
                        jspinner.addChangeListener(this);
                        //jslider.addMouseListener(this);
                        //jspinner.addMouseListener(this);

                        gd.showDialog();
                        if (gd.wasCanceled()){
                            angle=curAngle;
                            return false;
                        }
                            angle = jslider.getValue();
                            curAngle=angle;
                            rotateRoi(angle);

                        return true;
                }

                public synchronized void stateChanged(ChangeEvent e) {
                    if(e.getSource() instanceof JSlider)
                     {angle = jslider.getValue();
                    //System.out.println("jslider_angle="+angle);
                }
                    else if(e.getSource() instanceof JSpinner)
                     {angle = ((Integer)jspinner.getValue()).intValue();
                     //System.out.println("jspinner_angle="+angle);
                 }
                     double difangle = angle-oldangle;
                     imp.updateAndDraw();
                     rotate2DRoi(difangle);
                     oldangle=angle;
                }
    public void insertUpdate(DocumentEvent e){
    }
    public void removeUpdate(DocumentEvent e){
    }
    public void changedUpdate(DocumentEvent e){
    }
    void rotate2DRoi(double angle) {
        Graphics2D g2D = (Graphics2D)g;
        Polygon p =new Polygon();
        double x, y, ax=0.0, ay=0.0, d2, d, width, height, xcenter, ycenter;
        double arg, a, argt;
        int type=0, npoints=4;
            if (roi!=null) {
                    p = roi.getPolygon();
                    npoints=p.npoints;
                    Rectangle r = roi.getBounds();
                    x=(r.x);
                    y=(r.y);
                    width=(r.width);
                    height=(r.height);
                    type=roi.getType();
            } else {
                    x=(0);
                    y=(0);
                    width=(imp.getWidth());
                    height=(imp.getHeight());
                    type=0;
            }
            xcenter=x+width/2;
            ycenter=y+height/2;
            xcenter*=mag;
            ycenter*=mag;
            //System.out.println("p.npoints="+p.npoints);
            double[] xa = new double[npoints];
            double[] ya = new double[npoints];
            int[] xam = new int[npoints];
            int[] yam = new int[npoints];
            AffineTransform at = g2D.getTransform();
            for (int i=0; i<npoints; i++){
                xa[i] = p.xpoints[i]*mag;
                ya[i] = p.ypoints[i]*mag;
                xam[i]=(int)Math.round(xa[i]);
                yam[i]=(int)Math.round(ya[i]);
            }
            g2D.translate(-srcRect.x*mag, -srcRect.y*mag);
            g2D.rotate(angle*(Math.PI/180.0), xcenter, ycenter);
            g2D.setColor(Roi.getColor());
            g2D.drawPolygon(xam, yam, npoints);
            g2D.translate(srcRect.x*mag, srcRect.y*mag);
    }

    void rotateRoi(double angle) {
        Polygon p =new Polygon();
        double x, y, ax=0.0, ay=0.0, d2, d, width, height, xcenter, ycenter;
        double arg, a, argt;
        int type=0, npoints=4;
            if (roi!=null) {
                    p = roi.getPolygon();
                    npoints=p.npoints;
                    Rectangle r = roi.getBounds();
                    x=(r.x);
                    y=(r.y);
                    width=(r.width);
                    height=(r.height);
                    type=roi.getType();
            } else {
                    x=(0);
                    y=(0);
                    width=(imp.getWidth());
                    height=(imp.getHeight());
                    type=0;
            }
            if (type==Roi.RECTANGLE) type = Roi.POLYGON;
            if (type==Roi.OVAL) type = Roi.FREEROI;
            xcenter=x+width/2;
            ycenter=y+height/2;
            //System.out.println("p.npoints="+p.npoints);
            double[] xcoord = new double[npoints];
            double[] ycoord = new double[npoints];
            double[] xa = new double[npoints];
            double[] ya = new double[npoints];
            int[] xam = new int[npoints];
            int[] yam = new int[npoints];

            for (int i=0; i<npoints; i++){
            xa[i] = p.xpoints[i];
            ya[i] = p.ypoints[i];
            arg=angle*(Math.PI/180.0);
            double sa = Math.sin(arg);
            double ca = Math.cos(arg);
            xcoord[i] = xa[i]*ca-ya[i]*sa - xcenter*ca+ycenter*sa + xcenter;
            ycoord[i] = xa[i]*sa+ya[i]*ca - xcenter*sa-ycenter*ca + ycenter;
            xam[i]=(int)Math.round(xcoord[i]);
            yam[i]=(int)Math.round(ycoord[i]);
            }
            try {
                roi = new PolygonRoi(xam, yam, npoints, type);
                imp.setRoi(roi);
            } catch (Exception ex) {
                System.out.println("IllegalArgumentException: Roi.COMPOSITE ="+type);
            }
    }

}
